/* Copyright (C) 2011, 2012, 2013 by Bundesanstalt für Gewässerkunde
 * Software engineering by Intevation GmbH
 *
 * This file is Free Software under the GNU AGPL (>=v3)
 * and comes with ABSOLUTELY NO WARRANTY! Check out the
 * documentation coming with Dive4Elements River for details.
 */

package org.dive4elements.river.artifacts.uinfo.inundationduration;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.apache.log4j.Logger;
import org.dive4elements.artifactdatabase.data.StateData;
import org.dive4elements.artifactdatabase.state.Facet;
import org.dive4elements.artifacts.Artifact;
import org.dive4elements.artifacts.CallContext;
import org.dive4elements.artifacts.CallMeta;
import org.dive4elements.artifacts.common.utils.XMLUtils.ElementCreator;
import org.dive4elements.river.artifacts.D4EArtifact;
import org.dive4elements.river.artifacts.states.AddTableDataHelper;
import org.dive4elements.river.artifacts.states.DefaultState;
import org.dive4elements.river.artifacts.uinfo.UINFOArtifact;
import org.w3c.dom.Element;

/** State in which to fetch years for sedminent load calculation. */
public class LoadSingleYearSelectState extends DefaultState {
    private static final long serialVersionUID = 1L;
    /** The log used in this class. */
    private static Logger log = Logger.getLogger(LoadSingleYearSelectState.class);

    private InundationDurationAccess access;

    /**
     * The default constructor that initializes an empty State object.
     */
    public LoadSingleYearSelectState() {
    }

    /** Year Select Widget. */
    @Override
    protected String getUIProvider() {
        return "common.state.load_single_year_select";
    }

    @Override
    protected void appendItems(final Artifact artifact, final ElementCreator creator, final String name, final CallContext context, final Element select) {
        final String dataKey = "singleyear";
        try {
            if (dataKey.equals(name)) {

                final AddTableDataHelper helper = new AddTableDataHelper(creator, select, "year", context.getMeta()); // pinKeycolum; nicht zu verweschseln mit
                                                                                                                      // datakey..artifact.

                helper.addColumn(0, "pinfrom", "60", "common.client.ui.selection", "ICON", "CENTER", "from");
                helper.addColumn(1, "year", "60", "year", "INTEGER", "LEFT", null);
                final Map<String, StateData> data = super.getData();
                final UedauernPropertiesHelper properties = UedauernPropertiesHelper.getInstance(this.access.getRiverName());

                for (final String year : properties.getEinzeljahre()) {
                    final Map<String, String> row = new HashMap<>();
                    row.put("year", year);
                    helper.addRow(row);

                }
                helper.submitMapToXml();
            }
        }
        catch (final IllegalArgumentException iae) {
            iae.printStackTrace();
        }
    }

    @Override
    public Object computeInit(final D4EArtifact artifact, final String hash, final Object context, final CallMeta meta, final List<Facet> facets) {
        if (artifact instanceof UINFOArtifact) {
            this.access = new InundationDurationAccess((UINFOArtifact) artifact);
        }
        return super.computeInit(artifact, hash, context, meta, facets);
    }

}